#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE
 # Copyright (C) 2021 KylinSoft Co., Ltd.
 #
 # Authors:
 #  Yang Min yangmin@kylinos.cn
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import GLib

import os
import dbus
import signal
import dbus.service
import dbus.mainloop.glib

import subprocess

DBUSNAME = "com.kylin.screen.rotation"

PROWHITELIST=["/usr/bin/screenMonitorGeneral", "/usr/bin/screenMonitor", "/usr/bin/d-feet", "/usr/bin/ukui-control-center"]

class DemoException(dbus.DBusException):
    _dbus_error_name = "com.kylin.screen.rotation.Exception"

class TestObject(dbus.service.Object):
    INTERFACE = "com.kylin.screen.rotation.interface"
    def __init__(self, system_bus, mainloop):
        dbus.service.Object.__init__(self, system_bus,
                                     "/")
        self.mainloop = mainloop

        command = "find /sys/devices/platform/ -type f -name 'gsensor_info'"
        processGW = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell = True)
        self.ifaceGW = bytes.decode(processGW.stdout.read().strip())

        self.ifaceI2C0 = "/sys/bus/iio/devices/iio:device0"
        gname = self.ifaceI2C0 + "/name"

        #### 深中微整机接口路径
        self.flagSZW = False
        if (os.path.exists(gname)):
            with open(gname) as fp:
                for line in fp.readlines():
                    if (line.strip() == "sc7a20"): # GSensor name
                        self.flagSZW = True

        #### 706整机TR3265接口路径
        self.flagTR3265 = False
        if (os.path.exists(gname)):
            with open(gname) as fp:
                for line in fp.readlines():
                    if (line.strip() == "lis2dw12"): # GSensor name
                        self.flagTR3265 = True

        ## 采样频率规划中应该作为可配置选项，目前使用默认值
        freqPath = self.ifaceI2C0 + "/sampling_frequency"
        if os.path.exists(freqPath):
            os.system("echo 100 > " + freqPath)


    def CheckEnv(self, pid):
        file_path = "/proc/{}/environ".format(pid)

        with open(file_path, "r") as fp:
            for line in fp.readline().replace('\0', '\n').split("\n"):
                if line == "" or "=" not in line:
                    continue
                mid = line.split("=")[0]
                if mid == "LD_PRELOAD" or mid == "LD_LIBRARY_PATH" or mid == "LD_AUDIT":
                    print("{} dbus method control, env forbidden\n".format(pid))
                    return False
        return True

    def ISAllowedCaller(self, pid):
        exec_path = "/proc/{}/exe".format(pid)
        real_exec_path = os.readlink(exec_path)
        for path in PROWHITELIST:
            if real_exec_path == path:
                return True
        print("{} dbus method control, operation not permitted\n".format(real_exec_path))
        return False


    #@dbus.service.method(INTERFACE, in_signature = 's', out_signature = 'as')
    #def HelloWorld(self, hello_message):
    #    print(str(hello_message))
    #    return ["Hello", "from example-service.py", "with unique name",
    #            bus.get_unique_name()]

    #@dbus.service.method(INTERFACE, in_signature = '', out_signature = 'ss')
    #def GetTuple(self):
    #    return ("Hello Tuple", " from example-service.py")

    @dbus.service.method(INTERFACE,
                         in_signature = '', out_signature = 's', sender_keyword='sender')
    def GetCurrentScreenStatus(self, sender=None):
        ### 门禁系统
        bus = dbus.SystemBus().get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
        pid = dbus.Interface(bus, 'org.freedesktop.DBus').GetConnectionUnixProcessID(sender)

        ## check call env
        if not self.CheckEnv(pid):
            return "dbus method control, env forbidden"

        ## check caller white list
        if not self.ISAllowedCaller(pid):
            return "dbus method control, operation not permitted"

        kernelStatus = ""
        ### 长城整机
        if os.path.exists(self.ifaceGW):
            processGW2 = subprocess.Popen("cat " + self.ifaceGW, stdout=subprocess.PIPE, shell = True)
            result = bytes.decode(processGW2.stdout.read())
            gwx = gwy = gwz = -1
            for l in result.split("\n"):
                if ":" not in l:
                    continue
                if "axis_x" in l:
                    gwx = int(l.split(":")[1])
                if "axis_y" in l:
                    gwy = int(l.split(":")[1])
                if "axis_z" in l:
                    gwz = int(l.split(":")[1])
            if gwx >= 768 and gwx <= 1280:
                kernelStatus = "left"
            elif gwx >= 2816 and gwx <= 3328:
                kernelStatus = "right"
            else:
                kernelStatus = "normal"

        ### 深中微整机
        elif os.path.exists(self.ifaceI2C0) and self.flagSZW:

            processSZWx = subprocess.Popen("cat /sys/bus/iio/devices/iio\:device0/in_accel_x_raw", stdout=subprocess.PIPE, shell = True)
            processSZWx.wait()
            valuex = processSZWx.stdout.read()
            processSZWy = subprocess.Popen("cat /sys/bus/iio/devices/iio\:device0/in_accel_y_raw", stdout=subprocess.PIPE, shell = True)
            processSZWy.wait()
            valuey = processSZWy.stdout.read()

            if valuex != b'' and valuey != b'':
                x = int(valuex.decode().strip())
                y = int(valuey.decode().strip())

                if x in range(-520, 520) and y > 900:
                    kernelStatus = 'left'
                elif x in range(-520, 520) and y < -900:
                    kernelStatus = 'right'
                elif y in range(-920, 920) and x > 400:
                    kernelStatus = 'inverted'
                else:
                    kernelStatus = 'normal'

#                if x < -360 and y in range(-305, 265):
#                    kernelStatus = 'left'
#                elif x > 360 and y in range(-305, 265):
#                    kernelStatus = 'right'
#                else:
#                    kernelStatus = 'normal'

        ### 706整机TR3265
        elif os.path.exists(self.ifaceI2C0) and self.flagTR3265:

            processTRx = subprocess.Popen("cat /sys/bus/iio/devices/iio\:device0/in_accel_x_raw", stdout=subprocess.PIPE, shell = True)
            processTRx.wait()
            valuex = processTRx.stdout.read()
            processTRy = subprocess.Popen("cat /sys/bus/iio/devices/iio\:device0/in_accel_y_raw", stdout=subprocess.PIPE, shell = True)
            processTRy.wait()
            valuey = processTRy.stdout.read()

            if valuex != b'' and valuey != b'':
                x = int(valuex.decode().strip())
                y = int(valuey.decode().strip())

                if x in range(770, 1100):
                    kernelStatus = 'left'
                elif x in range(-1100, -670):
                    kernelStatus = 'right'
                else:
                    kernelStatus = 'normal'

        return kernelStatus 

    #@dbus.service.method(INTERFACE, in_signature = '', out_signature = '')
    #def RaiseException(self):
    #    raise DemoException("The RaiseException method does what you might "
    #                        'expect')

    #@dbus.service.method(INTERFACE, in_signature = '', out_signature = '')
    #def Exit(self):
    #    self.mainloop.quit()

if __name__ == '__main__':
    os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"

    dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)
    mainloop = GLib.MainLoop()

    bus = dbus.SystemBus()
    name = dbus.service.BusName(DBUSNAME, bus)
    signal.signal(signal.SIGINT, lambda : mainloop.quit())
    obj = TestObject(bus, mainloop)

    print("Running kyas_system_service...\n")
    mainloop.run()
    print("Ending kyas_system_service...\n")
